@@ -1,11 +1,10 @@
 """
-"Grab bag" of utility functions.
+Grab bag of utility functions.
 """
 
 import abc
 import base64
 import collections
-import copy
 import json
 import logging
 import random
@@ -15,15 +14,16 @@ import time
 import uuid
 import warnings
 import zipfile
-
-# for type checking
 from collections.abc import Mapping
+from copy import deepcopy
 from io import BytesIO, StringIO
 
 import numpy as np
 
+from .iteration import chain
+
 # use our wrapped types for wider version compatibility
-from .typed import Any, Iterable, List, Union
+from .typed import ArrayLike, Dict, Iterable, NDArray, Optional, Set, Union, float64
 
 # create a default logger
 log = logging.getLogger("trimesh")
@@ -441,7 +441,7 @@ def vector_to_spherical(cartesian):
     return spherical
 
 
-def spherical_to_vector(spherical):
+def spherical_to_vector(spherical: ArrayLike) -> NDArray[float64]:
     """
     Convert an array of `(n, 2)` spherical angles to `(n, 3)` unit vectors.
 
@@ -1279,13 +1279,13 @@ def comment_strip(text, starts_with="#", new_line="\n"):
     return result
 
 
-def encoded_to_array(encoded):
+def encoded_to_array(encoded: Union[Dict, ArrayLike]) -> NDArray:
     """
     Turn a dictionary with base64 encoded strings back into a numpy array.
 
     Parameters
     ------------
-    encoded : dict
+    encoded
       Has keys:
         dtype: string of dtype
         shape: int tuple of shape
@@ -1294,7 +1294,7 @@ def encoded_to_array(encoded):
 
     Returns
     ----------
-    array: numpy array
+    array
     """
 
     if not isinstance(encoded, dict):
@@ -1384,59 +1384,6 @@ def type_named(obj, name):
     raise ValueError("Unable to extract class of name " + name)
 
 
-def chain(*args: Union[Iterable[Any], Any, None]) -> List[Any]:
-    """
-    A less principled version of `list(itertools.chain(*args))` that
-    accepts non-iterable values, filters `None`, and returns a list
-    rather than yielding values.
-
-    If all passed values are iterables this will return identical
-    results to `list(itertools.chain(*args))`.
-
-
-    Examples
-    ----------
-
-    In [1]: list(itertools.chain([1,2], [3]))
-    Out[1]: [1, 2, 3]
-
-    In [2]: trimesh.util.chain([1,2], [3])
-    Out[2]: [1, 2, 3]
-
-    In [3]: trimesh.util.chain([1,2], [3], 4)
-    Out[3]: [1, 2, 3, 4]
-
-    In [4]: list(itertools.chain([1,2], [3], 4))
-      ----> 1 list(itertools.chain([1,2], [3], 4))
-      TypeError: 'int' object is not iterable
-
-    In [5]: trimesh.util.chain([1,2], None, 3, None, [4], [], [], 5, [])
-    Out[5]: [1, 2, 3, 4, 5]
-
-
-    Parameters
-    -----------
-    args
-      Will be individually checked to see if they're iterable
-      before either being appended or extended to a flat list.
-
-
-    Returns
-    ----------
-    chained
-      The values in a flat list.
-    """
-    # collect values to a flat list
-    chained = []
-    # extend if it's a sequence, otherwise append
-    [
-        chained.extend(a) if is_sequence(a) else chained.append(a)
-        for a in args
-        if a is not None
-    ]
-    return chained
-
-
 def concatenate(
     a, b=None
 ) -> Union["trimesh.Trimesh", "trimesh.path.Path2D", "trimesh.path.Path3D"]:  # noqa: F821
@@ -1516,16 +1463,30 @@ def concatenate(
         if _STRICT:
             raise E
 
+    metadata = {}
+    try:
+        [metadata.update(deepcopy(m.metadata) for m in is_mesh)]
+    except BaseException:
+        pass
+
     # create the mesh object
-    return trimesh_type(
+    result = trimesh_type(
         vertices=vertices,
         faces=faces,
         face_normals=face_normals,
         vertex_normals=vertex_normals,
         visual=visual,
+        metadata=metadata,
         process=False,
     )
 
+    try:
+        result._source = deepcopy(is_mesh[0].source)
+    except BaseException:
+        pass
+
+    return result
+
 
 def submesh(
     mesh, faces_sequence, repair=True, only_watertight=False, min_faces=None, append=False
@@ -1620,8 +1581,11 @@ def submesh(
             faces=faces,
             face_normals=np.vstack(normals),
             visual=visual,
+            metadata=deepcopy(mesh.metadata),
             process=False,
         )
+        appended._source = deepcopy(mesh.source)
+
         return appended
 
     if visuals is None:
@@ -1634,12 +1598,14 @@ def submesh(
             faces=f,
             face_normals=n,
             visual=c,
-            metadata=copy.deepcopy(mesh.metadata),
+            metadata=deepcopy(mesh.metadata),
             process=False,
         )
         for v, f, n, c in zip(vertices, faces, normals, visuals)
     ]
 
+    [setattr(r, "_source", deepcopy(mesh.source)) for r in result]
+
     if only_watertight or repair:
         # fill_holes will attempt a repair and returns the
         # watertight status at the end of the repair attempt
@@ -1918,7 +1884,9 @@ def decompress(file_obj, file_type):
     if file_type.endswith("bz2"):
         import bz2
 
-        return {file_obj.name[:-4]: wrap_as_stream(bz2.open(file_obj, mode="r").read())}
+        # get the file name if we have one otherwise default to "archive"
+        name = getattr(file_obj, "name", "archive1234")[:-4]
+        return {name: wrap_as_stream(bz2.open(file_obj, mode="r").read())}
     if "tar" in file_type[-6:]:
         import tarfile
 
@@ -1957,7 +1925,7 @@ def compress(info, **kwargs):
     return compressed
 
 
-def split_extension(file_name, special=None):
+def split_extension(file_name, special=None) -> str:
     """
     Find the file extension of a file name, including support for
     special case multipart file extensions (like .tar.gz)
@@ -2015,17 +1983,20 @@ def triangle_strips_to_faces(strips):
     """
 
     # save the length of each list in the list of lists
-    lengths = np.array([len(i) for i in strips])
+    lengths = np.array([len(i) for i in strips], dtype=np.int64)
     # looping through a list of lists is extremely slow
     # combine all the sequences into a blob we can manipulate
-    blob = np.concatenate(strips)
+    blob = np.concatenate(strips, dtype=np.int64)
 
-    # preallocate and slice the blob into rough triangles
-    tri = np.zeros((len(blob) - 2, 3), dtype=np.int64)
-    for i in range(3):
-        tri[: len(blob) - 3, i] = blob[i : -3 + i]
-    # the last triangle is left off from the slicing, add it back
-    tri[-1] = blob[-3:]
+    # slice the blob into rough triangles
+    tri = np.array([blob[:-2], blob[1:-1], blob[2:]], dtype=np.int64).T
+
+    # if we only have one strip we can do a *lot* less work
+    # as we keep every triangle and flip every other one
+    if len(strips) == 1:
+        # flip in-place every other triangle
+        tri[1::2] = np.fliplr(tri[1::2])
+        return tri
 
     # remove the triangles which were implicit but not actually there
     # because we combined everything into one big array for speed
@@ -2064,7 +2035,7 @@ def triangle_fans_to_faces(fans):
         np.transpose([fan[0] * np.ones(len(fan) - 2, dtype=int), fan[1:-1], fan[2:]])
         for fan in fans
     ]
-    return np.concatenate(faces, axis=1)
+    return np.concatenate(faces)
 
 
 def vstack_empty(tup):
@@ -2301,7 +2272,7 @@ def decode_text(text, initial="utf-8"):
     Try to decode byte input as a string.
 
     Tries initial guess (UTF-8) then if that fails it
-    uses chardet to try another guess before failing.
+    uses charset_normalizer to try another guess before failing.
 
     Parameters
     ------------
@@ -2323,12 +2294,11 @@ def decode_text(text, initial="utf-8"):
         text = text.decode(initial)
     except UnicodeDecodeError:
         # detect different file encodings
-        import chardet
+        from charset_normalizer import detect as charset_normalizer_detect
 
         # try to detect the encoding of the file
-        # only look at the first 1000 characters otherwise
-        # for big files chardet looks at everything and is slow
-        detect = chardet.detect(text[:1000])
+        # only look at the first 1000 characters for speed
+        detect = charset_normalizer_detect(text[:1000])
         # warn on files that aren't UTF-8
         log.debug(
             "Data not {}! Trying {} (confidence {})".format(
@@ -2407,7 +2377,11 @@ def is_ccw(points, return_all=False):
     return ccw, area, centroid
 
 
-def unique_name(start, contains, counts=None):
+def unique_name(
+    start: Optional[str],
+    contains: Union[Set, Mapping, Iterable],
+    counts: Optional[Dict] = None,
+):
     """
     Deterministically generate a unique name not
     contained in a dict, set or other grouping with
